/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozCC
 *
 * The Initial Developer of the Original Code is
 * Nathan R. Yergler.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Nathan R. Yergler <nathan@yergler.net>
 *
 * ***** END LICENSE BLOCK ***** 
 *
 * $Id: license.js,v 1.10 2004/01/26 16:31:15 nathan Exp $
 *
 */

include ('chrome://mozcc/content/ccrdf.js');

var g_attrsSeen = new Array();

// parseRdf
/*  Takes a list of strings, each containing an RDF block;
 *  returns a list of RDF() objects.
 */
function parseRdf(segments) {

   var parsed = new Array();
   var parser;

   for (idx=0; idx < segments.length; idx++) {
        parser = new jlRDF();
	parser.parseRDF(segments[idx], noop, [])

        parsed.push(parser);

   } // for each segment

   for (foo=0; foo < parsed.length; foo++) {
     alert(parsed[foo].getTriples());
   }

   return parsed;

   function noop(args) {
      return true;
   }
      
} // parseRdf

// parent and child are both treeitems
function appendChild(parent, child) {

   var treechildren = parent.getElementsByTagName('treechildren');

   if (treechildren.length == 0) {
      // add the treechildren element
      children = document.createElement("treechildren");
      parent.appendChild(children);
   } else {
     children = treechildren[0];
   }

   children.appendChild(child);
}

function createChild(contents, container) {

   var child = document.createElement("treeitem");
   child.setAttribute("container", container);
   child.setAttribute("open", true);


   var row = document.createElement("treerow");
   
   for (i=0; i < contents.length; i++) {
      var cell = document.createElement("treecell");
      cell.setAttribute("label", contents[i]);

      // check if this cell is a link
      if ((/http:/gi).test(contents[i])) {
         cell.setAttribute("properties", "isLink");
         cell.setAttribute("tooltiptext", "double click to open URI");
      }

      row.appendChild(cell);
   }

   child.appendChild(row);

   return child;
}

function rpart(instr) {
   return instr.split('/').reverse()[0]
}

function isSubObject(object) {
 return false;
  return (object.indexOf('genid:') > -1);
}

function isType(predicate) {
  return (predicate.indexOf('#') > -1);
}

function beenSeen(item) {
   // checks for membership in g_attrsSeen

   for (i=0; i < g_attrsSeen.length; i++) {
      if (g_attrsSeen[i] == item) return true;
   }

   return false;

}

function setSeen(item) {
   // mark an attr as seen
   g_attrsSeen.push(item);
}

function showAttrs(rdfDict, parent) {

   arcs = rdfDict.keys();

   for (a = 0; a < arcs.length; a++) {

      // get each target value for this arc label
      targets = rdfDict.getAll(arcs[a].Value);

      for (t = 0; t < targets.length; t++) {

        // check if this is a type declaration
        if (isType(arcs[a].Value)) continue;

        // check if this is a subobject
        if (isSubObject(arcs[a].Value)) {
         if (!(beenSeen(arcs[a].Value))) {
           setSeen(arcs[a].Value);

           attr = createChild([rpart(arcs[a].Value)], true);
           appendChild(parent, attr);
  
           // showAttrs(rdfDict.subObject(arcs[a]), attr);
         }
        } else {
           attr = createChild([rpart(arcs[a].Value), 
                               targets[t].Value], false);
           appendChild(parent, attr);
        }

      } // for each target

   } // for each attribute

}

function addAttrGrid(rdfDict, parent) {

  // setup the grid 
  grid = document.createElement("listbox");

  // setup the headers
  headers = document.createElement("listhead");

  head1 = document.createElement("listheader");
  head1.setAttribute("label", "property");
  head2 = document.createElement("listheader");
  head2.setAttribute("label", "value");

  headers.appendChild(head1);
  headers.appendChild(head2);

  grid.appendChild(headers);

  // setup the columns
  cols = document.createElement("listcols");

  col1 = document.createElement("listcol");
  col2 = document.createElement("listcol");

  cols.appendChild(col1);
  cols.appendChild(col2);

  grid.appendChild(cols);

  // get a list of keys to iterate through
  arcs = rdfDict.keys();
  for (a=0; a < arcs.length; a++) {
    targets = rdfDict.getAll(arcs[a].Value);
    for (t=0; t < targets.length; t++) {

      // create the row container
      row = document.createElement("listitem");
  
      // create the cells
      cell1 = document.createElement("listcell");
      cell1.setAttribute("label", rpart(arcs[a].Value));

      cell2 = document.createElement("listcell");
      cell2.setAttribute("label", targets[t].Value);

      // assemble the row
      row.appendChild(cell1);
      row.appendChild(cell2);

      // attach the row
      grid.appendChild(row);

    } // for each target
  } // for each arc


  // insert the grid
  parent.appendChild(grid);

} // addAttrGrid

function addWork(work) {
   works_cont = window.document.getElementById("works_container");

   // add the work descriptor
   var work_child = document.createElement("description");
   work_child.setAttribute("value", work.about());

   works_cont.appendChild(work_child);

   // add the work properties
   addAttrGrid(work, works_cont);

} // addWord

function addLicense(license) {
   license_cont = window.document.getElementById("licenses_container");

   // add the license descriptor
   var license_desc = document.createElement("description");
   license_desc.setAttribute("value", license.about());

   license_cont.appendChild(license_desc);

   // add the license images
   hbox = document.createElement("hbox");
   makeLicenseImages(license, hbox);
   license_cont.appendChild(hbox);

   // add the license properties 
   addAttrGrid(license, license_cont);

} // addLicense

function onLoadLicense(event) {

   status_msg = window.document.getElementById("rdf_status");
   rdf_url    = window.document.getElementById("rdf_url");
   rdf_tree   = window.document.getElementById("rdf_info");
   rdf_children = window.document.getElementById("rdf_children");

   rdfStrings = extractRdf(window.arguments[0]); 

   // update the status message
   status_msg.value = "Parsed " + rdfStrings.length + " RDF segments from";
   rdf_url.value = window.arguments[0].URL;

   // make the RDF tab the default
   window.document.getElementById("parsed_info").selectedIndex = 2;

   // for each segment, add a node
   for (s = 0; s < rdfStrings.length; s++) {

      var cc = new ccRdf();
      cc.parse(rdfStrings[s], rdf_url.value);

      // update the RDF tab
      segment = createChild(['Segment ' + (s+1)], true);
      rdf_children.appendChild(segment);

      // for work information, add a node
      works = cc.works();
      for (w = 0; w < works.length; w++) {
         work = createChild([works[w].about()], true);
         appendChild(segment, work);

         // add a node for each work attribute
         attrs = works[w].keys();
         g_attrsSeen = new Array();
         showAttrs(works[w], work);

         // update the works tab (if necessary)
         addWork(works[w]);
      }  // for each work

      // for each license add a node
      licenses = cc.licenses();
      for (l=0; l < licenses.length; l++) {
         license = createChild([licenses[l].about()], true);
	 appendChild(segment, license);

         // add a node for each attribute
         attrs = licenses[l].keys();
         g_attrsSeen = new Array();
         showAttrs(licenses[l], license);

         // update the licenses tab (if necessary)
         addLicense(licenses[l]);
      } // for each license
   } // for each segment

  return true;
}

/* dblClickRow
 *   Fired when user double-clicks a row in the tree.  Determines
 *   which column was clicked, checks if the cell is a link and if
 *   so opens the URI in a new window.
 */
function dblClickRow(event) {

   rdf_tree     = window.document.getElementById("rdf_info");
   rdf_children = window.document.getElementById("rdf_children");

   pred_col = window.document.getElementById("predicate");

   // determine which column was clicked
   if (event.pageX <= pred_col.boxObject.width) {
      celltext = rdf_tree.view.getCellText(rdf_tree.currentIndex, "predicate");
   } else {
      celltext = rdf_tree.view.getCellText(rdf_tree.currentIndex, "object");
   }

   // check if the clicked area is a link
   if ((/http:/gi).test(celltext)) {
      openUrl(celltext);
   }
   
}


function noop(args) {
   return true;
}

/* openUrl(startpage)
 *   Opens the given url (startpage) in a new browser window.
 *   Adapted from browser.js; copyright, license and source at
 *  http://lxr.mozilla.org/mozilla/source/browser/base/content/browser.js
 */
function openUrl(startpage) {

   var charsetArg = new String();
   var handler = Components.classes['@mozilla.org/commandlinehandler/general-startup;1?type=browser'];
   handler = handler.getService();
   handler = handler.QueryInterface(Components.interfaces.nsICmdLineHandler);

   var url = handler.chromeUrlForTask;
   var wintype = document.firstChild.getAttribute('windowtype');
 
   // if and only if the current window is a browser window and 
   // it has a document with a character
   // set, then extract the current charset menu setting from 
   // the current document and use it to
   // initialize the new browser window...
   if (window && (wintype == "navigator:browser") && 
       window._content && window._content.document)
   {
     var DocCharset = window._content.document.characterSet;
     charsetArg = "charset="+DocCharset;
 
     //we should "inherit" the charset menu setting in a new window
     window.openDialog(url, "_blank", "chrome,all,dialog=no", 
                       startpage, charsetArg);
   }
   else // forget about the charset information.
   {
     window.openDialog(url, "_blank", "chrome,all,dialog=no", startpage);
   }

}

